#include "buffer.h"

#include <cstdio>

#define BUFFER_OFFSET(i) ((char *)NULL + (i))

template <typename T>
GLenum Buffer<T>::getGLEnum(BufferType type)
{
   GLenum typeEnum;
    
    switch(m_type)
    {
        case VBO :
           typeEnum = GL_ARRAY_BUFFER;
           break;
        case EBO :
           typeEnum = GL_ELEMENT_ARRAY_BUFFER;
           break;
        case UBO :
           typeEnum = GL_UNIFORM_BUFFER;
           break;
    }
    
    return typeEnum;
}

template <typename T>
Buffer<T>::BufferEditor::BufferEditor(Buffer *b)
{
    if(b->isDynamic())
    {
        GLenum m_typeEnum = getGLEnum(b->getType());
        glBindBuffer(m_typeEnum, b->getId());
        ptr = (T*)glMapBuffer(m_typeEnum, GL_WRITE_ONLY);
    }
    else
    {
        fprintf(stderr, "Buffer data can't be edited, this buffer is static\n");
        ptr = NULL;
    }
}

template <typename T>
Buffer<T>::BufferEditor::~BufferEditor()
{
    glUnmapBuffer(m_typeEnum);
    glBindBuffer(m_typeEnum, 0);
}

template <typename T>
Buffer<T>::Buffer(const std::vector<T> &data, BufferType type, bool isDynamic) :
   m_type(type),
   m_isDynamic(isDynamic)
{
    // TODO : allow stream draw
    GLenum draw_type = isDynamic ? GL_DYNAMIC_DRAW : GL_STATIC_DRAW;
    GLenum typeEnum = getGLEnum(m_type);
    glGenBuffers(1, m_id);
    glBindBuffer(typeEnum, m_id);
    glBufferData(typeEnum, data.size() * sizeof(T), data.data(), draw_type);
    glBindBuffer(typeEnum, 0);
}

template <typename T>
void Buffer<T>::setVertexAttrib(int location, int nbComponents, int offset, int instanceDivisor)
{
    if(m_type == VBO)
    {
        glBindBuffer(GL_ARRAY_BUFFER, m_id);
        glEnableVertexAttribArray(location);
        if(instanceDivisor)
            glVertexAttribDivisor(location, instanceDivisor);
        glVertexAttribPointer(location, nbComponents, GL_FLOAT, GL_FALSE, sizeof(T), BUFFER_OFFSET(offset));
        glBindBuffer(GL_ARRAY_BUFFER, 0);
    }
    else
        fprintf(stderr, "WTF\n");
}

template <typename T>
Buffer<T>::~Buffer()
{
    glDeleteBuffers(1, m_id);
}
